/*
   FNT - SQL-backend for PostgreSQL - function query

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <libpq-fe.h>
#include <stdlib.h>
#include <string.h>

#include "../include/global.h"

#include "../include/types.h"
#include "../include/error.h"
#include "../include/plugin.h"
#include "../include/sql.h"

#include "main.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

/*** file scope functions **********************************************/

/*** public functions **************************************************/

DLLEXPORT fnt_sql_query_t *
query (fnt_sql_connect_t * sql_connect, const char *query_string)
{
    fnt_sql_query_t *ret_value;
    PGresult *result;

    if ((sql_connect == NULL) || (query_string == NULL))
	return NULL;

    ret_value = g_try_malloc (sizeof (fnt_sql_query_t));
    if (ret_value == NULL)
	return NULL;

    result = PQexec ((PGconn *) sql_connect->db_handle, query_string);
    if (result == NULL)
    {
	g_free (ret_value);
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			"%s",
			PQerrorMessage ((PGconn *) sql_connect->db_handle));
	return NULL;
    }

    switch (PQresultStatus (result))
    {
    case PGRES_EMPTY_QUERY:
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			_("Pgsql - query string empty"));
	PQclear (result);
	g_free (ret_value);
	return NULL;
	break;
    case PGRES_BAD_RESPONSE:
    case PGRES_FATAL_ERROR:
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			"%s", PQresultErrorMessage (result));
	PQclear (result);
	g_free (ret_value);
	return NULL;
	break;
    case PGRES_NONFATAL_ERROR:
    case PGRES_COMMAND_OK:
	ret_value->is_result_present = false;
	ret_value->num_fields = 0;
	ret_value->num_rows = (unsigned long long) PQcmdTuples (result);
	break;
    case PGRES_TUPLES_OK:
	ret_value->is_result_present = true;
	ret_value->num_fields = PQnfields (result);
	ret_value->num_rows = PQntuples (result);
	break;
    default:
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			_("Psql - unhandled response from server"));
	PQclear (result);
	g_free (ret_value);
	return NULL;
	break;
    }
    ret_value->handle = (void *) result;
    ret_value->__current_row = -1;
    return ret_value;
}
